		TITLE	CCS.ASM
		PAGE	60,132

;----------------------------------------------------------------;
;  CCS.CTL - Allows user to choose his CONFIG.SYS options        ;
;  at boot time.  All possible options are put into one          ;
;  CONFIG.SYS file.  User has menu selection at boot time.       ;
;  The user should put the line DEVICE=CCS.CTL [wait] [def]      ;
;  just before the first configuration option.  [wait] is the    ;
;  number of seconds to wait for a key to be hit before the      ;
;  default configuration is used.  If [wait] is omitted, the     ;
;  menu is put up automatically.  [def] is a single alphabetic   ;
;  character for the default menu item selection.  If [def] is   ;
;  too large, too small or omitted, the default selection will   ;
;  be "A".                                                       ;
;  CCS.CTL is a character device driver.  When it is executed    ;
;  by DOS at boot up, it accesses the CONFIG.SYS file in memory  ;
;  and modifies it prior to further processing by DOS.  CCS.CTL  ;
;  uses the DEVICE=OPTION <string> expression to delimit the     ;
;  different configuration options the user can choose from.     ;
;  DEVICE=ENDOPT is used to mark the end of the option.          ;
;  <string> is a descriptive expression to be placed in the      ;
;  menu.  At present, 52 characters is the longest <string>      ;
;  which can be stored.  Longer strings are truncated.  The user ;
;  should put the line DEVICE=DONECSS at the end of the          ;
;  CONFIG.SYS files                                              ;
;  The device also installs itself to handle requests for input. ;
;  This allows a program to access the selection chosen by the   ;
;  user at boot time.  This can be used to select the matching   ;
;  AUTOEXEC option automatically.  The device can be accessed    ;
;  through the file "CCS103XX"                                   ;
;  One restriction is that it can handle 100 lines per option.   ;
;  If you need more, change MAXLINES to a larger number and      ;
;  recompile.  Or you could rewrite the command line processing  ;
;  to include an option to specify the maximum number of lines   ;
;  per option.                                                   ;
;                                                                ;
;  Example CONFIG.SYS file                                       ;
;    ... Lines common to all configurations                      ;
;    DEVICE=CCS.CTL 3 A                                          ;
;    DEVICE=OPTION B [Descriptive text]                          ;
;    ... Lines of configuration B                                ;
;    DEVICE=ENDOPT                                               ;
;    ... Lines common to all configurations                      ;
;    DEVICE=OPTION A [Descriptive text]                          ;
;    ... Lines of configuration A                                ;
;    DEVICE=ENDOPT                                               ;
;    ... Lines common to all configurations                      ;
;    DEVICE=OPTION C [Descriptive text]                          ;
;    ... Lines of configuration C                                ;
;    DEVICE=ENDOPT                                               ;
;    ... Lines common to all configurations                      ;
;    DEVICE=DONECCS                                              ;
;                                                                ;
;  Example AUTOEXEC.BAT                                          ;
;    ... Lines common to all configurations                      ;
;    OPTPICK --  this program accesses the device to set up an   ;
;            --  environment variable CCS103                     ;
;    if "%CCS103%"=="" goto noconfig                             ;
;    goto config%CCS103%                                         ;
;    :noconfig                                                   ;
;    echo cannot find configuration, check config.sys and        ;
;    echo autoexec.bat                                           ;
;    goto done                                                   ;
;    :configA                                                    ;
;    ... Lines for configuration A                               ;
;    GOTO done                                                   ;
;    :configB                                                    ;
;    ... Lines for configuration B                               ;
;    GOTO done                                                   ;
;    :configC                                                    ;
;    ... Lines for configuration C                               ;
;    :done                                                       ;
;    ... Lines common to all configurations                      ;
;                                                                ;
;  Author - Soh Kok Hong                                         ;
;----------------------------------------------------------------;

;----------------------------------------------------------------;
;                                                                ;
;  To make file CCS.CTL:                                         ;
;    tasm /m2 CCS                                                ;
;    tlink /x CCS                                                ;
;    exe2bin CCS.EXE CCS.CTL                                     ;
;                                                                ;
;----------------------------------------------------------------;

_TEXT		SEGMENT PUBLIC 'CODE'
		ASSUME	CS:_TEXT,DS:_TEXT,ES:_TEXT,SS:_TEXT

		ORG	0H

;-------- DEVICE HEADER --------;

pointer		dd	-1
attribute	dw	8000H
dev_strategy	dw	STRATEGY
dev_interrupt	dw	INTERRUPT
dev_name	db	"CCS103XX"

CR		EQU	13
LF		EQU	10
LFLF		EQU	0A0AH
SPACE		EQU	32
TAB		EQU	09
BELL		EQU	07
ESC_KEY		EQU	27
UP_KEY		EQU	48H
DN_KEY		EQU	50H

UPPER_LEFT	EQU	0C9H
UPPER_RIGHT	EQU	0BBH
LOWER_LEFT	EQU	0C8H
LOWER_RIGHT	EQU	0BCH
HORIZ		EQU	0CDH
VERT		EQU	0BAH
CENTER_LEFT	EQU	0C7H
CENTER_RIGHT	EQU	0B6H
HORIZ1		EQU	0C4H

DONE		EQU	0100H
UNKNOWN_CMD	EQU	8003H

MAXLINES	EQU	100	;maximum number of lines per option
MAXWIDTH	EQU	60	;width of box
INV_CURSOR	EQU	2000H	;invisible cursor mode
NORM		EQU	07
HIGHMODE	EQU	70H
;-----------------------------;

REQUEST_HEADER	STRUC

HEADER_LENGTH	DB	?
UNIT_CODE	DB	?
COMMAND_CODE	DB	?
STATUS		DW	?
RESERVED	DQ	?

REQUEST_HEADER	ENDS

INIT		STRUC

INIT_HEADER	DB	(TYPE REQUEST_HEADER) DUP(?)
UNITS		DB	?
ENDING_OFFSET	DW	?
ENDING_SEGMENT	DW	?
ARGUMENTS_ADDR	DD	?

INIT		ENDS

INPUT_STRUC	STRUC

INPUT_HEADER	DB	(TYPE REQUEST_HEADER) DUP(?)
MEDIA		DB	?
DEST_ADDR	DD	?
INPUT_COUNT	DW	?
SECTOR_START	DW	?

INPUT_STRUC	ENDS

;------------ store address of request header ----------;
req_off		dw	?
req_seg		dw	?

;------------ location to store option selected ----------;
option_sel	db	?

;------------ CODE AREA ------------;

;********** strategy procedure of device driver *************;
STRATEGY	proc	far

		mov	cs:req_seg,es		;Store away request header
		mov	cs:req_off,bx		; address
		ret

STRATEGY	endp

;********** interrupt procedure of device **********;
INTERRUPT	proc	far

		push	ax		; save registers first
		push	bx
		push	ds
		push	si
		pushf

		lds	bx,dword ptr cs:req_off	;retrieve request header ptr
		mov	ah,COMMAND_CODE[bx]
		cmp	ah,4			;is it input command
		jz	input_cmd		;if yes, do input
		cmp	ah,0			;is it init command
		jz	init_cmd		;if yes, do initialization

		mov	word ptr STATUS[bx],UNKNOWN_CMD
		jmp	exit_here

done_cmd:	lds	bx,dword ptr cs:req_off
		mov	word ptr STATUS[bx],DONE	;return status to DOS
exit_here:	popf					;restore registers
		pop	si
		pop	ds
		pop	bx
		pop	ax
		ret
INTERRUPT	ENDP

;********** This section handles input command **********;
input_cmd:	cmp	INPUT_COUNT[bx],0
		jz	zero_byte

		push	ds
		lds	si,DEST_ADDR[bx]	;get destination address ptr
		mov	al,cs:option_sel
		mov	[si],al
		pop	ds
zero_byte:	mov	INPUT_COUNT[bx],1
		jmp	done_cmd

;********** This section handles initialization *******;
init_cmd:	cli
		mov	cs:save_ss,ss		;save stack segment
		mov	cs:save_sp,sp		;save stack pointer
		mov	ax,cs			;set up new stack segment
		mov	ss,ax
		mov	sp,0fffeh		;set up new stack pointer
		sti
		push	cx
		push	dx
		push	es
		push	di
		push	bp
		mov	ENDING_OFFSET[bx],offset init_cmd
		mov	ENDING_SEGMENT[bx],cs	;setup resident portion
		cld				;clear direction flag

		les	di,ARGUMENTS_ADDR[bx]	;get pointer to
						;CONFIG.SYS buffer

		push	cs			;point ds to code data
		pop	ds

		mov	ah,0fh			;get video mode
		int	10h
		xor	ah,ah			;set video mode
		int	10h			; to clear screen

		xor	dx,dx			;set cursor position
		call	set_cur

		mov	dx,offset copyright_msg
		call	print_string
		xor	bp,bp			;used to store delay

		mov	ah,30h			;get dos version
		int	21h
		mov	dosver,al

proc_cmdline:	call	skip_terminators	;shift di to terminators
						; 0,CR,LF,SPACE,TAB
proc_cmd1:	call	skip_past_ws		;shift di to non-white space
						; 0,SPACE,TAB.
		mov	al,es:[di]
		call	is_eol			;if no arguments then jump
		jz	proc_config

		cmp	al,"0"			;check for number
		jc	store_sel		;if smaller, store select
		cmp	al,"9"
		ja	store_sel		;if larger, store select
		and	al,0fh			;else calculate and store delay
		cbw
		xchg	ax,bp
		mov	cx,10
		mul	cx
		add	bp,ax
		inc	di
		jmp	proc_cmd1
store_sel:	call	is_letter
		jnz	proc_config
		mov	default_sel,al		;last parameter, move on

proc_config:	mov	delay,bp
		call	skip_past_eol	;move di past LF
		mov	buffer_start,di
proc_next_opt:	mov	cx,MAXLINES	;limit to MAXLINES lines per option
proc_next_line:	mov	bp,di		;store away pointer to start of line
		push	cx

		mov	si,offset option_ccs	;search for DEVICE=OPTION
		mov	cx,OPTION_CCS_LENGTH
		rep	cmpsb
		jz	found_option
		mov	di,bp
		mov	si,offset doneccs	;look for end marker
		mov	cx,DONECCS_LENGTH
		rep	cmpsb
		jnz	not_doneccs
		jmp	doneccs_found

not_doneccs:	pop	cx			;cannot find OPTION on current
		call	skip_past_eol		; line, try next line
		loop	proc_next_line

disp_error:	mov	dx,offset error_msg1	;too many lines in common
		jmp	print_error		; option, exit
found_option:	pop	cx
		mov	cx,MAXLINES
		inc	option_ctr
		cmp	option_ctr,17		;Only 17 options available,
						; because of limited number
						; of lines on screen to
						; display.
		jna	option_cnt_ok		; otherwise, display error.
		mov	dx,offset error_msg2
		jmp	print_error

option_cnt_ok:	mov	si,bp			;get pointer to start of line
		mov	byte ptr es:[si],"#"	;replace DEVICE marker "D" with
						; special marker
		;search for option letter
		call	skip_past_ws
		mov	al,es:[di]
		call	is_letter
		jnz	no_opt_letter

		;confirm whitespace after option letter.
		inc	di
		mov	bl,al			;save option letter
		mov	al,es:[di]
		call	is_space
		jz	is_opt_letter
		call	is_eol
		jz	is_opt_letter

no_opt_letter:	mov	dx,offset error_msg4	;no option letter found,
print_error:	call	print_string		; so print error message
		lds	bx,dword ptr cs:req_off	;reset ending offset of
		mov	ENDING_OFFSET[bx],0	; resident portion of driver.
		jmp	restore

		;check for repeat option letter
is_opt_letter:	mov	al,bl			;restore option letter
		call	get_selptr
		cmp	bl,option_ctr
		jc	repeat_option
		mov	option_letter[bx-1],al
		jmp	proc_next_lin2

repeat_option:	mov	repeat_letter,al
		mov	dx,offset error_msg5	;option letter repeated,
		jmp	print_error		; so print error message and
						; exit

		;inside option now, start looking for DEVICE=ENDOPT
proc_next_lin2:	call	skip_past_eol
		mov	bp,di
		push	cx

		mov	si,offset endopt_ccs
		mov	cx,ENDOPT_CCS_LENGTH
		rep	cmpsb
		jz	found_endopt
		pop	cx
		loop	proc_next_lin2
		mov	dx,offset error_msg3
		jmp	print_error

found_endopt:	pop	cx
		mov	si,bp
		mov	byte ptr es:[si],"^"

		call	skip_past_eol	;prepare to process next option
		jmp	proc_next_opt

doneccs_found:	pop	cx
		inc	di		;This change was made to
					; ensure no error message for 
					; DOS 2.x and 3.x.
		mov	buffer_end,di	;store ptr to end of CONFIG.SYS buffer
		mov	di,bp
		mov	byte ptr es:[di],"#"

		call	get_cur		;get cursor state
		mov	curmode,cx	;store cursor mode
		mov	curposn,dx	;store cursor position

		xor	bx,bx
		cmp	default_sel,0	;if default_sel was not set up
					; use first option letter as default
		jz	no_default
		mov	al,default_sel	;check if default
		call	get_selptr
		cmp	bl,option_ctr
		jc	do_delay

		mov	warning_letter,al
		mov	al,option_letter
		mov	default_letter,al
		mov	dx,offset warning_msg1	;print warning message
		call	print_string		; that selected default option
		xor	bl,bl			; is no available

		;after this point, default_sel is a pointer into
		; option_letter pointing to default selection
do_delay:	mov	default_sel,bl		;set up default_sel pointer
no_default:	cmp	delay,0			;check delay
		jz	nodelay
		mov	dx,offset prompt_msg
		call	print_string

		xor	ah,ah		;get time in BP using BIOS
		int	1ah
		mov	bp,dx
wait_delay:	mov	ah,01		;test for key hit
		int	16h
		jnz	key_hit
		xor	ah,ah
		int	1ah
		sub	dx,bp
		cmp	dx,18		;a second up?
		jc	wait_delay	;if no, jump
		add	bp,dx		;save new time
		dec	delay		;decrement delay by a second
		jnz	wait_delay

		mov	dx,curposn	;reset cursor position
		call	set_cur
		mov	al,option_letter[bx]
		mov	default_sel,al
		mov	dx,offset config_sel_msg
		call	print_string
		mov	default_sel,bl
		jmp	enddelay

key_hit:	xor	ah,ah		;read key into AX
		int	16h
		cmp	al,ESC_KEY	;if ESC was pressed, go use
		jnz	nodelay		; default selection.
		jmp	enddelay
nodelay:	mov	dx,curposn
		call	set_cur
		inc	dl		;shift cursor 1 column right
		add	dh,2		; 2 rows down
		mov	curposn,dx

		mov	dx,offset query_msg
		call	print_string

		xor	bx,bx
		mov	di,buffer_start
		mov	cl,option_ctr
		xor	ch,ch
next_label:	push	cx
		mov	al,"#"
		call	get_to_char

		add	di,OPTION_CCS_LENGTH+3	;point di to OPTION message
		mov	al,option_letter[bx]
		mov	maxsel,al		;setup option letter
		mov	dx,offset sel_msg
		call	print_string
		call	setup_menu

		inc	bl			;next option letter
		call	skip_past_eol
		pop	cx
		loop	next_label

		mov	dx,offset lowbox_msg
		call	print_string
		call	get_cur
		mov	final_curp,dx		;store final cursor position

		mov	cx,INV_CURSOR		;make cursor invisible
		mov	ah,1
		int	10h

		mov	dx,curposn
		mov	bl,default_sel		;get default selection
		add	dh,bl			;point dh to correct row
		call	highlight		;highlight current selection
		mov	al,option_ctr		;adjust maxsel_ptr to 1 less
		dec	al			; max number of options
		mov	maxsel_ptr,al

wait_key:	xor	ah,ah
		int	16h
		cmp	al,CR
		jz	hitCR

		cmp	ah,UP_KEY		;is UP key pressed?
		jnz	notUP			;if not, goto notUP
		cmp	bl,0			;if yes, is current select 1st
						; letter?
		jz	wrongsel		;if yes, go ring BELL
		dec	bl			;otherwise, adjust current sel
		call	unhigh			;shift highlight up
		dec	dh
		call	highlight
		jmp	wait_key

wrongsel:	mov	al,BELL
		call	write_tty
		jmp	wait_key

notUP:		cmp	ah,DN_KEY		;is DN key pressed?
		jnz	notDN			;if not, goto notDN
		cmp	bl,maxsel_ptr		;if yes, is current select max?
		jz	wrongsel		; if yes, ring bell.
		inc	bl			;otherwise, adjust current sel
		call	unhigh			;shift highlight down
		inc	dh
		call	highlight
		jmp	wait_key

notDN:		and	al,5fh			;convert to uppercase
		call	is_letter
		jnz	wrongsel

		mov	cl,bl		;check if available option
		call	get_selptr
		cmp	bl,option_ctr
		xchg	bl,cl		;get old selection ptr into bl
					; and current selection ptr into cl
		jnc	wrongsel	;jump if wrong letter selected

		call	unhigh		;process selection here.
		xchg	bl,cl		;exchange old and new pointers
		sub	cl,bl
		sub	dh,cl		;adjust dh to new row
		call	highlight

hitCR:		mov	cx,curmode	;reset cursor mode from invisible
		mov	ah,01
		int	10h

		mov	dx,final_curp
		call	set_cur

enddelay:	mov	al,option_letter[bx]
		mov	option_sel,al
		mov	di,buffer_start		;set up to copy to CONFIG.SYS
						; buffer
		mov	cl,bl
		inc	cl
		call	copy_unsel		;copy unselected lines

		mov	al,"^"			;copy selected lines
		mov	marker,al
		call	copylines
		call	skip_past_eol

		mov	cl,option_ctr		;copy unselected lines
		sub	cl,bl
		call	copy_unsel

done_copy:	mov	di,buffer_start
		mov	cx,buffer_end	;prepare to pad rest of buffer
		sub	cx,di

		cmp	dosver,04	;is it DOS 4.x?
		jc	notdos4		;if yes,
		mov	ax,LFLF		; pad with linefeeds
		jmp	dos4
notdos4:	dec	di		;changed in version 1.02 to prevent
		xor	ax,ax		; "Unrecognized command in CONFIG.SYS"
					; message from appearing.

dos4:		shr	cx,1
		rep	stosw
		jae	restore
		stosb

restore:	pop	bp		;restore everything
		pop	di
		pop	es
		pop	dx
		pop	cx
		cli
		mov	ss,cs:save_ss
		mov	sp,cs:save_sp
		sti
		jmp	done_cmd

;********** is_space procedure **********;
; check if al is a whitespace 0,TAB,SPACE
; returns zero flag set if it is.
is_space	proc	near

		cmp	al,0
		jz	rtn
		cmp	al,SPACE
		jz	rtn
		cmp	al,TAB
rtn:		ret
is_space	endp

;********** is_letter procedure **********;
; check if al is a letter [A-Z].
; returns zero flag set if it is.
is_letter	proc	near

		cmp	al,"A"
		jc	rtn
		cmp	al,"Z"
		ja	rtn
		cmp	al,al
		jmp	rtn
is_letter	endp

;********** is_eol procedure **********;
; check if al is a EOL char - CR,LF
; returns zero flag set if it is
is_eol		proc	near

		cmp	al,LF
		jz	rtn
		cmp	al,CR
		jmp	rtn
is_eol		endp

;********** get_selptr procedure **********;
; get pointer into option_letter pointing
; to selected option letter in al
get_selptr	proc	near

		xor	bx,bx
cmp_option:	cmp	al,option_letter[bx]
		jz	letter_found
		inc	bl
		cmp	bl,option_ctr
		jc	cmp_option
letter_found:	ret
get_selptr	endp

;********** copy_unsel procedure **********;
; copy unselected lines in buffer
; uses di, al, cl, si
; adjusts di to new buffer location.
copy_unsel	proc	near

		cmp	byte ptr es:[di],"#"
		jz	gettosel
		mov	al,"#"
		mov	marker,al
		call	copylines
gettosel:	call	skip_past_eol
		dec	cl
		jz	endcopy_unsel
		mov	al,"^"
		call	get_to_char
		call	skip_past_eol
		jmp	copy_unsel
endcopy_unsel:	ret
copy_unsel	endp

;********** copylines procedure **********;
;copies lines in buffer to buffer_start
;until marker character is reached
;uses si, di, al
;adjusts di to new buffer location
copylines	proc	near

		mov	si,di
		mov	di,buffer_start
		push	ds
		mov	ax,es
		mov	ds,ax
		mov	al,cs:marker
ck_marker:	cmp	byte ptr [si],al	;look for marker
		jnz	copybyte
		pop	ds
		mov	buffer_start,di
		mov	di,si
		ret
copybyte:	cmp	byte ptr [si],LF
		movsb
		jnz	copybyte		;keep copying until LF char
		jmp	ck_marker
copylines	endp

;********** print_string procedure **********;
print_string	proc	near

		mov	ah,09
		int	21h
		ret
print_string	endp

;********** get_cur procedure **********;
get_cur		proc	near

		push	ax
		push	bx
		mov	ah,03
		xor	bh,bh
		int	10h
		pop	bx
		pop	ax
		ret
get_cur		endp

;********** set_cur procedure **********;
set_cur		proc	near

		push	ax
		push	bx
		mov	ah,02
		xor	bh,bh
		int	10h
		pop	bx
		pop	ax
		ret
set_cur		endp

;********** skip_terminators procedure **********;
; incr DI until one of 0, CR, LF, SPACE, TAB
; appears in ES:[DI]
skip_terminators	proc	near

		push	ax
skip_term:	mov	al,es:[di]
		call	is_space
		jz	go_home
		call	is_eol
		jz	go_home
		inc	di
		jmp	skip_term
go_home:	pop	ax
		ret

skip_terminators	endp

;********** skip_past_ws procedure **********;
; this procedure increments di until es:[di] is not
; a 0, SPACE, or TAB
skws1:		inc	di
skip_past_ws	proc	near

		cmp	byte ptr es:[di],0
		jz	skws1
		cmp	byte ptr es:[di],SPACE
		jz	skws1
		cmp	byte ptr es:[di],TAB
		jz	skws1
		ret

skip_past_ws	endp

;********** skip_past_eol procedure **********;
; This procedure increments DI until it reaches the first
; non-LF character after a CRLF combination in ES:[DI]
skip_past_eol	proc	near

		push	ax
		push	cx
		mov	cx,130		;maximum line length
		mov	al,LF

		repne	scasb

		pop	cx
		pop	ax
		dec	di
clr_lf:		inc	di
		cmp	byte ptr es:[di],CR	;skip empty lines
		jz	clr_lf
		cmp	byte ptr es:[di],LF
		jz	clr_lf
		ret
skip_past_eol	endp

;********** get_to_char procedure **********;
; this procedure increments DI until ES:[DI] is
; pointing to the character in AL
get_to_char	proc	near

		cmp	es:[di],al
		jz	foundopt
		call	skip_past_eol
		jmp	get_to_char
foundopt:	ret
get_to_char	endp

;********** setup_menu procedure **********;
; procedure to setup a menu
; It also contains the write_tty procedure.
; This procedure reads the ES:[DI] buffer
; and writes the buffer to screen until
; a CR or LF is found.  It also replaces 0
; with a space.  The string is limited to
; MAXWIDTH - 7 characters in length
setup_menu	proc	near

		call	skip_past_ws
		mov	cx,MAXWIDTH-7
next_char:	mov	al,es:[di]
		call	is_eol
		jz	isLFCR
		inc	di		;di is not increment earlier to
					; allow processing of skip_past_eol
					; in parent procedure
		cmp	al,0
		jnz	notzero		;replace 0 with space
		mov	al,SPACE
notzero:	call	write_tty
		loop	next_char
isLFCR:		inc	cx
		mov	al,SPACE
pad_space:	call	write_tty
		loop	pad_space

		mov	al,VERT
		call	write_tty
		mov	al,CR
		call	write_tty
		mov	al,LF
write_tty:	mov	ah,0eh
		int	10h
		ret
setup_menu	endp

;********** unhighlight and highlight procedures **********;
unhigh		proc	near

		mov	attrib,NORM
		jmp	normal
unhigh		endp

highlight	proc	near

		mov	attrib,HIGHMODE
normal:		push	dx
		push	cx
		push	bx
		push	ax
		call	set_cur
		xor	bh,bh
		mov	cx,MAXWIDTH-2
high1:		mov	ah,8
		int	10h
		push	cx
		mov	cx,1
		mov	bl,attrib
		mov	ah,9
		int	10h
		pop	cx
		inc	dl
		call	set_cur
		loop	high1
		pop	ax
		pop	bx
		pop	cx
		pop	dx
		ret
highlight	endp

;----------- DATA area ----------;
save_ss		dw	?
save_sp		dw	?

dosver		db	?
option_ctr	db	0
option_letter	db	26 DUP (0)	;storage for option letters

marker		db	?	;marker character for copylines procedure
attrib		db	?
delay		dw	0		;default delay

buffer_start	dw	?		;point to start of CONFIG.SYS buffer
buffer_end	dw	?		;point to end of CONFIG.SYS BUFFER

curmode		dw	?
curposn		dw	?
final_curp	dw	?

option_ccs	db	"DOPTION"
OPTION_CCS_LENGTH	EQU	$-option_ccs
endopt_ccs	db	"DENDOPT"
ENDOPT_CCS_LENGTH	EQU	$-endopt_ccs
doneccs		db	"DDONECCS"
DONECCS_LENGTH		EQU	$-doneccs


sel_msg		db	VERT
		db	SPACE
maxsel		db	?
		db	". $"

copyright_msg	db	"CCS Version 1.03 "
		db	"Configuration Control System,",CR,LF
		db	"Freeware Copyright (C) 1991 by Soh Kok Hong",CR,LF,LF
		db	UPPER_LEFT
		db	(MAXWIDTH-2) DUP (HORIZ)
		db	UPPER_RIGHT,CR,LF,"$"

prompt_msg	db	VERT
		db	PROMPT_WIDTH DUP (SPACE)
prompt_nonws	db	"Hit any key for selection menu; "
		db	"ESC to boot with default"
PROMPT_WIDTH	equ	(MAXWIDTH-($-prompt_nonws)-2)/2
		db	(MAXWIDTH-($-prompt_msg)-1) DUP (SPACE)
		db	VERT,CR,LF,LOWER_LEFT
		db	(MAXWIDTH-2) DUP (HORIZ)
		db	LOWER_RIGHT,CR,LF,"$"

query_msg	db	VERT
		db	QUERY_WIDTH DUP (SPACE)
query_nonws	db	"Type LETTER or use UP or DOWN keys and ENTER"
QUERY_WIDTH	equ	(MAXWIDTH-($-query_nonws)-2)/2
		db	(MAXWIDTH-($-query_msg)-1) DUP (SPACE)
		db	VERT,CR,LF,CENTER_LEFT
		db	(MAXWIDTH-2) DUP (HORIZ1)
		db	CENTER_RIGHT,CR,LF,"$"

config_sel_msg	db	VERT
		db	CFG_SEL_WIDTH DUP (SPACE)
cfg_sel_nonws	db	"TIMED OUT: Default configuration "
default_sel	db	0
		db	" has been selected"
CFG_SEL_WIDTH	equ	(MAXWIDTH-($-cfg_sel_nonws)-2)/2
		db	(MAXWIDTH-($-config_sel_msg)-1) DUP (SPACE)
		db	VERT,CR,LF
lowbox_msg	db	LOWER_LEFT
		db	(MAXWIDTH-2) DUP (HORIZ)
		db	LOWER_RIGHT,CR,LF,"$"

maxsel_ptr	db	0

error_msg1	db	VERT
		db	ERROR_WIDTH1 DUP (SPACE)
error_msg1a	db	"ERROR: Cannot find DEVICE=OPTION or DEVICE=ENDOPT"
ERROR_WIDTH1	equ	(MAXWIDTH-($-error_msg1a)-2)/2
		db	(MAXWIDTH-($-error_msg1)-1) DUP (SPACE)
		db	VERT,CR,LF,LOWER_LEFT
		db	(MAXWIDTH-2) DUP (HORIZ)
		db	LOWER_RIGHT,CR,LF,"$"

error_msg2	db	VERT
		db	ERROR_WIDTH2 DUP (SPACE)
error_msg2a	db	"ERROR: Too many options - maximum 26 options"
ERROR_WIDTH2	equ	(MAXWIDTH-($-error_msg2a)-2)/2
		db	(MAXWIDTH-($-error_msg2)-1) DUP (SPACE)
		db	VERT,CR,LF,LOWER_LEFT
		db	(MAXWIDTH-2) DUP (HORIZ)
		db	LOWER_RIGHT,CR,LF,"$"

error_msg3	db	VERT
		db	ERROR_WIDTH3 DUP (SPACE)
error_msg3a	db	"ERROR: Too many lines in option - only 100 supported"
ERROR_WIDTH3	equ	(MAXWIDTH-($-error_msg3a)-2)/2
		db	(MAXWIDTH-($-error_msg3)-1) DUP (SPACE)
		db	VERT,CR,LF,LOWER_LEFT
		db	(MAXWIDTH-2) DUP (HORIZ)
		db	LOWER_RIGHT,CR,LF,"$"

error_msg4	db	VERT
		db	ERROR_WIDTH4 DUP (SPACE)
error_msg4a	db	"ERROR: Option letter missing in DEVICE=OPTION"
ERROR_WIDTH4	equ	(MAXWIDTH-($-error_msg4a)-2)/2
		db	(MAXWIDTH-($-error_msg4)-1) DUP (SPACE)
		db	VERT,CR,LF,LOWER_LEFT
		db	(MAXWIDTH-2) DUP (HORIZ)
		db	LOWER_RIGHT,CR,LF,"$"

error_msg5	db	VERT
		db	ERROR_WIDTH5 DUP (SPACE)
error_msg5a	db	"ERROR: Option letter "
repeat_letter	db	?
		db	" repeated"
ERROR_WIDTH5	equ	(MAXWIDTH-($-error_msg5a)-2)/2
		db	(MAXWIDTH-($-error_msg5)-1) DUP (SPACE)
		db	VERT,CR,LF,LOWER_LEFT
		db	(MAXWIDTH-2) DUP (HORIZ)
		db	LOWER_RIGHT,CR,LF,"$"

warning_msg1	db	VERT
		db	WARNING_WIDTH1 DUP (SPACE)
warning_msg1a	db	"Warning: Default option letter "
default_letter	db	?
		db	" used instead of "
warning_letter	db	?
WARNING_WIDTH1	equ	(MAXWIDTH-($-warning_msg1a)-2)/2
		db	(MAXWIDTH-($-warning_msg1)-1) DUP (SPACE)
		db	VERT,CR,LF,LOWER_LEFT
		db	(MAXWIDTH-2) DUP (HORIZ)
		db	LOWER_RIGHT,CR,LF
		db	UPPER_LEFT
		db	(MAXWIDTH-2) DUP (HORIZ)
		db	UPPER_RIGHT,CR,LF,"$"

_TEXT		ENDS
		END
